/*
 * $Id: TagUtils.java 421129 2006-07-12 05:13:54Z wsmoak $
 *
 * Copyright 1999-2006 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package it.eng.spago.taglib;

import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.RequestContainerAccess;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.ResponseContainerAccess;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.dispatching.TokenHandler;
import it.eng.spago.dispatching.service.DefaultRequestContext;
import it.eng.spago.dispatching.service.RequestContextIFace;
import it.eng.spago.message.MessageBundle;
import it.eng.spago.message.MessageBundleReference;
import it.eng.spago.services.ServiceCatalog;
import it.eng.spago.services.alias.AliasService;
import it.eng.spago.taglib.html.Constants;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.util.ContextScooping;
import it.eng.spago.util.HTMLUtils;

import java.io.IOException;
import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.BodyContent;

import org.apache.commons.validator.GenericValidator;

/**
 * Provides helper methods for JSP tags.
 *
 * @version $Rev: 421129 $
 * @since Struts 1.2
 */
public class TagUtils {
    
    /**
     * Constants used to ask the an envelop <code>characters</code> instead
     * of a specific attribute.
     */
    public static final String CHARACTERS_KEY = "@characters".intern();

	
	/**
     * The Singleton instance.
     * @since 1.3.5 Changed to non-final so it may be overridden, use at your own risk (you've been warned!!)
     */
    private static TagUtils instance = new TagUtils();

    /**
     * The message resources for this package. TODO We need to move the
     * relevant messages out of this properties file.
     */
    protected static MessageBundleReference messages =
        new MessageBundleReference(
            "org.apache.struts.taglib.LocalStrings");

    /**
     * Maps lowercase JSP scope names to their PageContext integer constant
     * values.
     */
    private static final Map scopes = new HashMap();

    /**
     * Initialize the scope names map and the encode variable with the
     * Java 1.4 method if available.
     */
    static {
        scopes.put("page", new Integer(PageContext.PAGE_SCOPE));
        scopes.put("request", new Integer(PageContext.REQUEST_SCOPE));
        scopes.put("session", new Integer(PageContext.SESSION_SCOPE));
        scopes.put("application", new Integer(PageContext.APPLICATION_SCOPE));
    }

    /**
     * Constructor for TagUtils.
     */
    protected TagUtils() {
        super();
    }

    /**
     * Returns the Singleton instance of TagUtils.
     */
    public static TagUtils getInstance() {
        return instance;
    }

    /**
     * Set the instance.
     * This blatently violates the Singleton pattern, but then some say Singletons are an anti-pattern.
     * @since 1.3.5 Changed to non-final and added setInstance() so TagUtils may be overridden, use at your own risk (you've been warned!!)
     * @param instance The instance to set.
     */
    public static void setInstance(TagUtils instance){
      TagUtils.instance = instance;
    }

    /**
     * Compute a set of query parameters that will be dynamically added to a
     * generated URL. The returned List is composed of <code>SourceBean</code>s;
     * every <code>SourceBean</code> rappresent a query string param and is
     * composed of two attribute: <code>name</code> and <code>value</code>. 
     * If no query parameters are identified, return an empty <code>List</code>.
     * 
     *
     * @param pageContext   PageContext we are operating in
     * @param paramId       Single-value request parameter name (if any)
     * @param paramName     <code>service context</code> data
     * 						containing single-value parameter value
     * @param paramScope    Scope containing data named by <code>paramName</code>
     * @param property      Property in <code>service context</code> that identify
     * 						a collection of parameters.
     * @param scope         Scope containing data named by <code>property</code>
     * @param transaction   Should we add our transaction control token?
     * 
     * @return List of <code>SourceBean</code> rappresenting query parameters. An
     * 						empty <code>List</code> if no query parameters are
     * 						found.
     * 
     * @throws JspException if we cannot look up the required data
     *                      
     */

    public List computeParameters(PageContext pageContext, String paramId,
        String paramName, String paramScope, 
        String property, String scope, boolean transaction)
        throws JspException {
        // Short circuit if no parameters are specified
        if ((paramId == null) && (property == null) && !transaction) {
            return (null);
        }

        // Locate the SourceBean list containing our multi-value attributes
        List paramsList = null;

        try {
            if (property != null) {
                paramsList = getInstance().lookupMultipleProperty
                		(pageContext, property, scope);
            }
        } catch (JspException e) {
            saveException(pageContext, e);
            throw e;
        }

        // Create a Map to contain our results from the multi-value parameters
        List results = null;

        if (paramsList != null) {
            results = new ArrayList(paramsList);
        } else {
            results = new ArrayList();
        }

        // Add the single-value parameter (if any)
        if ((paramId != null) && (paramName != null)) {
            Object paramValue = null;

            try {
                paramValue =
                    TagUtils.getInstance().lookupProperty(
                    		pageContext, paramName, paramScope);
            } catch (JspException e) {
                saveException(pageContext, e);
                throw e;
            }

            if (paramValue != null) {
                String paramString = null;

                if (paramValue instanceof String) {
                    paramString = (String) paramValue;
                } else {
                    paramString = paramValue.toString();
                }

                String sbName = (!GenericValidator.isBlankOrNull(property))?
                		property : paramId;
                SourceBean param = null;
                
				try {
					param = new SourceBean(sbName);
					param.setAttribute("NAME", paramId);
					param.setAttribute("VALUE", paramString);
				} catch (SourceBeanException e) {
				    TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
				    	"TagUtils::computeParameters:: exception creating sourcebean", e);
				}

                results.add(param);
            }
        }

        // Add our transaction control token (if requested)
        if (transaction) {
            HttpServletRequest httpRequest = (HttpServletRequest)pageContext.getRequest();
            RequestContainer requestContainer = RequestContainerAccess.getRequestContainer(httpRequest);
            String token = null;

            if (requestContainer != null) {
                token = (String)requestContainer.getSessionContainer()
                	.getPermanentContainer().getAttribute(TokenHandler.TRANSACTION_TOKEN_KEY);
                if (token == null) {
                    TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                        "TagUtils::computeParameters:: token nullo");
                }
            } else {
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                	"TagUtils::computeParameters:: requestContainer nullo");
            }

            if (token != null) {
            	try {
	            	SourceBean tokenParam = new SourceBean(TokenHandler.REQUEST_TOKEN_KEY);
	            	tokenParam.setAttribute("NAME", TokenHandler.REQUEST_TOKEN_KEY);
	            	tokenParam.setAttribute("VALUE", token);
	                results.add(tokenParam);
            	} catch (SourceBeanException e) {
                    TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                    	"TagUtils::computeParameters:: exception creating sourcebean", e);
            	}
            }
        }

        // Return the completed list
        return (results);
    }

    /**
     * @see #computeURLWithCharEncoding(PageContext, String, List, String, boolean, boolean)
     * 
     * @param pageContext
     * @param href
     * @param params
     * @param anchor
     * @param redirect
     * @return
     * @throws MalformedURLException
     * 
     */
    public String computeURL(PageContext pageContext, String href, List params,
        String anchor, boolean redirect)
        throws MalformedURLException, JspException {
        return this.computeURLWithCharEncoding(pageContext, href, params, anchor,
        		redirect, false);
    }

    /**
     * Compute a hyperlink URL based on the <code>href</code> parameter
     * that is not null. The returned URL will have already been passed to
     * <code>response.encodeURL()</code> for adding a session identifier.
     *
     * @param pageContext PageContext for the tag making this call
     * @param href        URL to be utilized unmodified (if specified)
     * @param params      List of <code>SourceBean</code> rappresenting
     * 					  parameters to be dynamically included (if any)
     * @param anchor      Anchor to be dynamically included (if any)
     * @param redirect    Is this URL for a <code>response.sendRedirect()</code>?
     * @return URL with session identifier
     * @throws java.net.MalformedURLException if a URL cannot be created for
     *                                        the specified parameters
     *                                        
     */
    public String computeURLWithCharEncoding(PageContext pageContext,
            String href, List params, String anchor, boolean redirect,
            boolean useLocalEncoding) throws MalformedURLException, JspException {
        return computeURLWithCharEncoding(pageContext, href, 
        	params, anchor, redirect, true, useLocalEncoding);
    }

    /**
     * @see #computeURLWithCharEncoding(PageContext, String, List, String, boolean, boolean, boolean)
     * 
     * @param pageContext
     * @param href
     * @param params
     * @param anchor
     * @param redirect
     * @param encodeSeparator
     * @return
     * @throws MalformedURLException
     */
    public String computeURL(PageContext pageContext, String href, List params,
        String anchor, boolean redirect, boolean encodeSeparator)
        throws MalformedURLException, JspException {
        return computeURLWithCharEncoding(pageContext, href, 
        		params, anchor, redirect, encodeSeparator, false);
    }
    
    public String computeURLWithCharEncoding(PageContext pageContext, String href, List params,
            String anchor, boolean redirect, boolean encodeSeparator,
            boolean useLocalSeparator)
            throws MalformedURLException, JspException {
    	
        return computeURLWithCharEncoding(pageContext, href, 
        		params, anchor, redirect, null, null, encodeSeparator, false);
    }

    /**
     * Compute a hyperlink URL based on the <code>href</code> parameter
     * that is not null. The returned URL will have already been passed to
     * <code>response.encodeURL()</code> for adding a session identifier.
     *
     * @param pageContext      PageContext for the tag making this call
     * @param href             URL to be utilized unmodified (if specified)
     * @param params           List of <code>SourceBean</code> rappresenting
     * 						   parameters to be dynamically included (if any)
     * @param anchor           Anchor to be dynamically included (if any)
     * @param redirect         Is this URL for a <code>response.sendRedirect()</code>?
     * @param alias            Logical name of a configured alias instead of th
     * 						   <code>href</code> parameter.
     * @param aliasRelative    String to append to the <code>alias</code> attribute
     * 						   if present.
     * @param encodeSeparator  This is only checked if redirect is set to
     *                         false (never encoded for a redirect).  If true,
     *                         query string parameter separators are encoded
     *                         as &gt;amp;, else &amp; is used.
     * @param useLocalEncoding If set to true, urlencoding is done on the
     *                         bytes of character encoding from
     *                         ServletResponse#getCharacterEncoding. Use UTF-8
     *                         otherwise.
     * @return URL with session identifier
     * @throws java.net.MalformedURLException if a URL cannot be created for
     *                                        the specified parameters
     *                                        
     */
    public String computeURLWithCharEncoding(PageContext pageContext,
        String href, List params, String anchor, boolean redirect,
        String alias, String aliasRelative,
        boolean encodeSeparator, boolean useLocalEncoding)
        throws MalformedURLException, JspException {
        String charEncoding = "UTF-8";

        if (useLocalEncoding) {
            charEncoding = pageContext.getResponse().getCharacterEncoding();
        }

        // TODO All the computeURL() methods need refactoring!
        // Validate that exactly one specifier was included

        if (href == null && alias == null) {
            throw new MalformedURLException(messages.getMessage(
            "computeURL.specifier"));
        }

        // Calculate the appropriate URL
        StringBuffer url = new StringBuffer();
        String configAlias = null;
        if (!GenericValidator.isBlankOrNull(alias)) {
        	try {
	        	AliasService aliasService = (AliasService)ServiceCatalog.getInstance()
	        		.getService(getRequestContext(pageContext), "ALIAS");
	        	
		        configAlias = aliasService.getAlias(alias);
        	} catch (Exception ex) {
        		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
        			"TagUtils::computeURLWithCharEncoding: Error retrieving 'alias service' or 'alias'", ex);
        		throw new JspException("Error retrieving alias", ex);
        	}
        }
        if (!GenericValidator.isBlankOrNull(configAlias)) {
        	url.append(configAlias);
        	if (!GenericValidator.isBlankOrNull(aliasRelative)) {
        		url.append(aliasRelative);
        	}
        } else if (href != null) {
            url.append(href);
        }

        // Add anchor if requested (replacing any existing anchor)
        if (anchor != null) {
            String temp = url.toString();
            int hash = temp.indexOf('#');

            if (hash >= 0) {
                url.setLength(hash);
            }

            url.append('#');
            url.append(this.encodeURL(anchor, charEncoding));
        }

        // Add dynamic parameters if requested
        if ((params != null) && (params.size() > 0)) {
            // Save any existing anchor
            String temp = url.toString();
            int hash = temp.indexOf('#');

            if (hash >= 0) {
                anchor = temp.substring(hash + 1);
                url.setLength(hash);
                temp = url.toString();
            } else {
                anchor = null;
            }

            // Define the parameter separator
            String separator = null;

            if (redirect) {
                separator = "&";
            } else if (encodeSeparator) {
                separator = "&amp;";
            } else {
                separator = "&";
            }

            // Add the required request parameters
            boolean question = temp.indexOf('?') >= 0;
            Iterator paramsIter = params.iterator();

            while (paramsIter.hasNext()) {
                SourceBean param = (SourceBean) paramsIter.next();
                String name = (String)param.getAttribute("NAME");
                Object value = param.getAttribute("VALUE");

                if (value == null) {
                    if (!question) {
                        url.append('?');
                        question = true;
                    } else {
                        url.append(separator);
                    }

                    url.append(this.encodeURL(name, charEncoding));
                    url.append('='); // Interpret null as "no value"
                } else if (value instanceof String) {
                    if (!question) {
                        url.append('?');
                        question = true;
                    } else {
                        url.append(separator);
                    }

                    url.append(this.encodeURL(name, charEncoding));
                    url.append('=');
                    url.append(this.encodeURL((String) value, charEncoding));
                } else /* Convert other objects to a string */
                 {
                    if (!question) {
                        url.append('?');
                        question = true;
                    } else {
                        url.append(separator);
                    }

                    url.append(this.encodeURL(name, charEncoding));
                    url.append('=');
                    url.append(this.encodeURL(value.toString(), charEncoding));
                }
            }

            // Re-add the saved anchor (if any)
            if (anchor != null) {
                url.append('#');
                url.append(this.encodeURL(anchor, charEncoding));
            }
        }

        // Perform URL rewriting to include our session ID (if any)
        // but only if url is not an external URL
        if ((href == null) && (pageContext.getSession() != null)) {
            HttpServletResponse response =
                (HttpServletResponse) pageContext.getResponse();

            if (redirect) {
                return (response.encodeRedirectURL(url.toString()));
            }

            return (response.encodeURL(url.toString()));
        }

        return (url.toString());
    }

    /**
     * URLencodes a string assuming the character encoding is UTF-8.
     *
     * @param url
     * @return String The encoded url in UTF-8
     */
    public String encodeURL(String url) {
        return encodeURL(url, "UTF-8");
    }

    /**
     * Use the new URLEncoder.encode() method from Java 1.4 if available, else
     * use the old deprecated version.  This method uses reflection to find
     * the appropriate method; if the reflection operations throw exceptions,
     * this will return the url encoded with the old URLEncoder.encode()
     * method.
     *
     * @param enc The character encoding the urlencode is performed on.
     * @return String The encoded url.
     */
    public String encodeURL(String url, String enc) {
        return HTMLUtils.encodeURL(url, enc);
    }

    /**
     * Filter the specified string for characters that are senstive to HTML
     * interpreters, returning the string with these characters replaced by
     * the corresponding character entities.
     *
     * @param value The string to be filtered and returned
     */
    public String filter(String value) {
        return HTMLUtils.filter(value);
    }

    /**
     * Return the form action converted into an action mapping path.  The
     * value of the <code>action</code> property is manipulated as follows in
     * computing the name of the requested mapping:
     *
     * <ul>
     *
     * <li>Any filename extension is removed (on the theory that extension
     * mapping is being used to select the controller servlet).</li>
     *
     * <li>If the resulting value does not start with a slash, then a slash is
     * prepended.</li>
     *
     * </ul>
     */
    public String getActionMappingName(String action) {
        String value = action;
        int question = action.indexOf("?");

        if (question >= 0) {
            value = value.substring(0, question);
        }

        int pound = value.indexOf("#");

        if (pound >= 0) {
            value = value.substring(0, pound);
        }

        int slash = value.lastIndexOf("/");
        int period = value.lastIndexOf(".");

        if ((period >= 0) && (period > slash)) {
            value = value.substring(0, period);
        }

        return value.startsWith("/") ? value : ("/" + value);
    }


    /**
     * Converts the scope name into its corresponding PageContext constant
     * value.
     *
     * @param scopeName Can be "page", "request", "session", or "application"
     *                  in any case.
     * @return The constant representing the scope (ie. PageContext.REQUEST_SCOPE).
     * @throws JspException if the scopeName is not a valid name.
     */
//    public int getScope(String scopeName)
//        throws JspException {
//        Integer scope = (Integer) scopes.get(scopeName.toLowerCase());
//
//        if (scope == null) {
//        	// TO DO
////            throw new JspException(messages.getMessage("lookup.scope", scope));
//        }
//
//        return scope.intValue();
//    }

    /**
     * Returns true if the custom tags are in XHTML mode.
     */
    public boolean isXhtml(PageContext pageContext) {
        String xhtml =
            (String) pageContext.getAttribute(Constants.XHTML_KEY,
                PageContext.PAGE_SCOPE);

        return "true".equalsIgnoreCase(xhtml);
    }

    public Object lookupProperty(PageContext pageContext, String property, String scopeName)
		throws JspException {
    	
    	return lookupProperty(pageContext, property, scopeName, true);
    }
    
    public RequestContextIFace getRequestContext(PageContext pageContext) {
        HttpServletRequest httpRequest = (HttpServletRequest)pageContext.getRequest();
        RequestContainer requestContainer = RequestContainerAccess.getRequestContainer(httpRequest);
        ResponseContainer responseContainer = ResponseContainerAccess.getResponseContainer(httpRequest);
        
        return new DefaultRequestContext(requestContainer, responseContainer);
    }
    
    public Object lookupProperty(PageContext pageContext, String property,
    		String scopeName, boolean emptyStringValue)	throws JspException {
    	// Metodo identico all'implementazione di ReloadField
        HttpServletRequest httpRequest = (HttpServletRequest)pageContext.getRequest();
        RequestContainer requestContainer = RequestContainerAccess.getRequestContainer(httpRequest);
        ResponseContainer responseContainer = ResponseContainerAccess.getResponseContainer(httpRequest);
      
        if (requestContainer == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "ReloadFieldTag::doStartTag:: requestContainer nullo");
        } // if (_content == null)
        
        if (responseContainer == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "ReloadFieldTag::doStartTag:: responseContainer nullo");
        } // if (_content == null)
     
        String scope = scopeName;
        if (GenericValidator.isBlankOrNull(scope)) {
        	scope = it.eng.spago.base.Constants.SERVICE_REQUEST;
        }
        
        //Default search the field Value in serviceRequest
        Object objectValue = ContextScooping.getScopedParameter(
        		requestContainer, responseContainer, property,
        		scope, null);

        String value = null;
        if (objectValue != null) {
        	if (objectValue instanceof String) {
        		value = (String)objectValue;
        	} else if (objectValue instanceof SourceBean) {
        		String chars = ((SourceBean)objectValue).getCharacters();
        		value = (GenericValidator.isBlankOrNull(chars))? objectValue.toString() : chars; 
        	} else {
        		value = objectValue.toString();
        	}
        }
        
        if (value == null && emptyStringValue) {
       			value = "";
        }
        
        return value;
    }
    
    public List lookupMultipleProperty(PageContext pageContext, String property,
        	String scopeName)
        	throws JspException {
    	
    	return lookupMultipleProperty(pageContext, property, scopeName, null, null, null);
    }
    
    public List lookupMultipleProperty(PageContext pageContext, String property,
    	String scopeName, String paramName, String paramValue, String listAttribute)
    	throws JspException {
    	// Metodo identico all'implementazione di ReloadField
        HttpServletRequest httpRequest = (HttpServletRequest)pageContext.getRequest();
        RequestContainer requestContainer = RequestContainerAccess.getRequestContainer(httpRequest);
        ResponseContainer responseContainer = ResponseContainerAccess.getResponseContainer(httpRequest);
      
        if (requestContainer == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "ReloadFieldTag::doStartTag:: requestContainer nullo");
        } // if (_content == null)
        
        if (responseContainer == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "ReloadFieldTag::doStartTag:: responseContainer nullo");
        } // if (_content == null)
     
        String scope = scopeName;
        if (GenericValidator.isBlankOrNull(scope)) {
        	scope = it.eng.spago.base.Constants.SERVICE_REQUEST;
        }

        //Default search the field Value in serviceRequest
        Object objectValue = ContextScooping.getScopedParameter(
        		requestContainer, responseContainer, property, 
        		scope, null);
        if ((objectValue instanceof List)
        	&& !GenericValidator.isBlankOrNull(paramName)
        	&& !GenericValidator.isBlankOrNull(paramValue)) {
        	
        	objectValue = getFilteredAttributes((List)objectValue,
        		paramName, paramValue);
        	
        	if (objectValue != null
        		&& objectValue instanceof SourceBean
        		&& listAttribute != null) {
        		
        		objectValue = ((SourceBean)objectValue).getAttributeAsList(
        			listAttribute);
        	}
        }
        List value = null;
        if (objectValue == null) {
   			value = new ArrayList();
        } else {
        	if (objectValue instanceof List) {
        		value = (List)objectValue;
        	} else {
        		// should you force conversion of objectValue to String?
        		value = new ArrayList();
      			value.add(objectValue);
        	}
        }
        
        return value;
    }
    
    private Object getFilteredAttributes(List attrs, String paramName, String paramValue) {
    	// I don't give the possibility to test a null parameter
        if (GenericValidator.isBlankOrNull(paramName)
        	|| GenericValidator.isBlankOrNull(paramValue)) {
        
        	return attrs;
        }
    	
    	List filteredAttrs = new ArrayList(attrs.size());
    	Iterator attrsIter = attrs.iterator();
    	while (attrsIter.hasNext()) {
    		Object attribute = attrsIter.next();
    		// if the attribute is not a SourceBean will not be returned
    		if (attribute instanceof SourceBean) {
    			Object attrParamValue = ((SourceBean)attribute).getAttribute(paramName); 
    			if (attrParamValue != null
    				&& (attrParamValue instanceof String)
    				&& ((String)attrParamValue).equalsIgnoreCase(paramValue)) {
    				
    				filteredAttrs.add(attribute);
    			}
    		}
    	}
    	 
    	return (filteredAttrs.size() == 1)?
    		filteredAttrs.iterator().next() :
    		filteredAttrs;
    }

    /**
     * Look up and return a message string, based on the specified
     * parameters.
     *
     * @param bundle      Name of the servlet context attribute for our
     *                    message resources bundle
     * @param key         Message key to be looked up and returned
     * @return message string
     * @throws JspException if a lookup error occurs (will have been saved in
     *                      the request already)
     *                      
     */
    public String message(String bundle, String key)
        throws JspException {
        return message(bundle, key, null);
    }
    
    /**
     * Look up and return a message string, based on the specified
     * parameters.
     * 
     * @param bundle
     * @param locale
     * @param key
     * @return
     */
    public String message(String bundle, Locale locale, String key) {
		String message = null;
		
    	if (GenericValidator.isBlankOrNull(bundle)) {
    		message = MessageBundle.getMessage(key, locale);
    	} else { 
    		message = MessageBundle.getMessage(key, bundle, locale);
    	}

        if ((message == null)) {
            // log missing key to ease debugging
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
            	"TagsUtils::message:: message nullo");
        }

        return message;
    }

    /**
     * Look up and return a message string, based on the specified
     * parameters.
     *
     * @param bundle      Name of the servlet context attribute for our
     *                    message resources bundle
     * @param key         Message key to be looked up and returned
     * @param args        Replacement parameters for this message
     * @return message string
     * @throws JspException if a lookup error occurs (will have been saved in
     *                      the request already)
     *                      
     */
	public String message(String bundle, String key, Object[] args)
        throws JspException {

		String message = null;
		
        if (args == null) {
        	if (GenericValidator.isBlankOrNull(bundle)) {
        		message = MessageBundle.getMessage(key);
        	} else { 
        		message = MessageBundle.getMessage(key, bundle);
        	}
        } else {
        	List argsList = Arrays.asList(args);
        	
        	if (GenericValidator.isBlankOrNull(bundle)) {
        		message = MessageBundle.getMessage(key, argsList);
        	} else {
        		message = MessageBundle.getMessage(key, bundle, argsList);
        	}
        }

        if ((message == null)) {
            // log missing key to ease debugging
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
            	"TagsUtils::message:: message nullo");
        }

        return message;
    }

    /**
     * Save the specified exception as a request attribute for later use.
     *
     * @param pageContext The PageContext for the current page
     * @param exception   The exception to be saved
     */
    public void saveException(PageContext pageContext, Throwable exception) {
        pageContext.setAttribute(Constants.EXCEPTION_KEY, exception,
            PageContext.REQUEST_SCOPE);
    }

    /**
     * Write the specified text as the response to the writer associated with
     * this page.  <strong>WARNING</strong> - If you are writing body content
     * from the <code>doAfterBody()</code> method of a custom tag class that
     * implements <code>BodyTag</code>, you should be calling
     * <code>writePrevious()</code> instead.
     *
     * @param pageContext The PageContext object for this page
     * @param text        The text to be written
     * @throws JspException if an input/output error occurs (already saved)
     */
    public void write(PageContext pageContext, String text)
        throws JspException {
        JspWriter writer = pageContext.getOut();

        try {
            writer.print(text);
        } catch (IOException e) {
            saveException(pageContext, e);
            throw new JspException(messages.getMessage("write.io", e.toString()));
        }
    }

    /**
     * Write the specified text as the response to the writer associated with
     * the body content for the tag within which we are currently nested.
     *
     * @param pageContext The PageContext object for this page
     * @param text        The text to be written
     * @throws JspException if an input/output error occurs (already saved)
     */
    public void writePrevious(PageContext pageContext, String text)
        throws JspException {
        JspWriter writer = pageContext.getOut();

        if (writer instanceof BodyContent) {
            writer = ((BodyContent) writer).getEnclosingWriter();
        }

        try {
            writer.print(text);
        } catch (IOException e) {
            saveException(pageContext, e);
            throw new JspException(messages.getMessage("write.io", e.toString()));
        }
    }
}
