/*
 * $Id: ErrorsTag.java 376841 2006-02-10 21:01:28Z husted $
 *
 * Copyright 2007 Engineering Ingegneria Informatica S.p.A.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package it.eng.spago.taglib.html;

import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.ResponseContainerAccess;
import it.eng.spago.error.EMFAbstractError;
import it.eng.spago.error.EMFErrorHandler;
import it.eng.spago.message.MessageBundle;
import it.eng.spago.taglib.TagUtils;
import it.eng.spago.tracing.TracerSingleton;

import java.util.Iterator;
import java.util.Locale;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.TagSupport;

import org.apache.commons.validator.GenericValidator;


/**
 * Custom tag that renders error messages if an appropriate request attribute
 * has been created.  The tag looks for a request attribute with a reserved
 * key, and assumes that it is either a String, a String array, containing
 * message keys to be looked up in the module's MessageResources, or an object
 * of type <code>org.apache.struts.action.ActionErrors</code>. <p> The
 * following optional message keys will be utilized if corresponding messages
 * exist for them in the application resources:
 *
 * <ul>
 *
 * <li><b>errors.header</b> - If present, the corresponding message will be
 * rendered prior to the individual list of error messages.</li>
 *
 * <li><b>errors.footer</b> - If present, the corresponding message will be
 * rendered following the individual list of error messages.</li>
 *
 * <li><b>errors.prefix</b> - If present, the corresponding message will be
 * rendered before each individual error message.</li>
 *
 * <li><b>errors.suffix</b> - If present, the corresponding message will be
 * rendered after each individual error message.</li>
 *
 * </ul>
 *
 * @version $Rev: 376841 $ $Date: 2005-08-21 19:08:45 -0400 (Sun, 21 Aug 2005)
 *          $
 */
public class ErrorsTag extends TagSupport {

	private static final long serialVersionUID = 1L;

    // ----------------------------------------------------------- Properties

    /**
     * The servlet context attribute key for our resources.
     */
    protected String bundle = Constants.MESSAGES_KEY;

    protected String category = null;
    protected String severity = null;

    /**
     * The name of the property for which error messages should be returned,
     * or <code>null</code> to return all errors.
     */
    protected String property = null;

    /**
     * The message resource key for errors header.
     */
    protected String header = null;

    /**
     * The message resource key for errors footer.
     */
    protected String footer = null;

    /**
     * The message resource key for errors prefix.
     */
    protected String prefix = null;

    /**
     * The message resource key for errors suffix.
     */
    protected String suffix = null;

	public void setCategory(String category) {
		this.category = category;
	}

	public void setSeverity(String severity) {
		this.severity = severity;
	}
    
    public String getBundle() {
        return (this.bundle);
    }

    public void setBundle(String bundle) {
        this.bundle = bundle;
    }

    public String getProperty() {
        return (this.property);
    }

    public void setProperty(String property) {
        this.property = property;
    }

    public String getHeader() {
        return (header == null) ? "errors.header" : header;
    }

    public void setHeader(String header) {
        this.header = header;
    }

    public String getFooter() {
        return (footer == null) ? "errors.footer" : footer;
    }

    public void setFooter(String footer) {
        this.footer = footer;
    }

    public String getPrefix() {
        return (prefix == null) ? "errors.prefix" : prefix;
    }

    public void setPrefix(String prefix) {
        this.prefix = prefix;
    }

    public String getSuffix() {
        return (suffix == null) ? "errors.suffix" : suffix;
    }

    public void setSuffix(String suffix) {
        this.suffix = suffix;
    }

    // ------------------------------------------------------- Public Methods

    /**
     * Render the specified error messages if there are any.
     *
     * @throws JspException if a JSP exception has occurred
     */
    public int doStartTag() throws JspException {
        if (pageContext == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                "ErrorsTag::doStartTag:: pageContext nullo");
            throw new JspException("pageContext nullo");
        } // if (_httpRequest == null)
        HttpServletRequest httpRequest = null;
        try {
            httpRequest = (HttpServletRequest)pageContext.getRequest();
        } // try
        catch (Exception ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "ErrorsTag::doStartTag::", ex);
            throw new JspException(ex.getMessage());
        } // catch (Exception ex)
        if (httpRequest == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                "ErrorsTag::doStartTag:: httpRequest nullo");
            throw new JspException("httpRequest nullo");
        } // if (_httpRequest == null)
        ResponseContainer responseContainer = ResponseContainerAccess.getResponseContainer(httpRequest);
        if (responseContainer == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                "ErrorsTag::doStartTag:: responseContainer nullo");
            throw new JspException("responseContainer nullo");
        } // if (responseContainer == null)
        EMFErrorHandler errorHandler = responseContainer.getErrorHandler();
    	
        if ((errorHandler == null) || errorHandler.isOK()) {
          return (EVAL_BODY_INCLUDE);
        }

        Locale userLocale = MessageBundle.getUserLocale();
        boolean headerPresent =
        	(TagUtils.getInstance().message(bundle, userLocale, getHeader()) != null);

        boolean footerPresent =
        	(TagUtils.getInstance().message(bundle, userLocale, getFooter()) != null);

        boolean prefixPresent =
        	(TagUtils.getInstance().message(bundle, userLocale, getPrefix()) != null);

        boolean suffixPresent =
        	(TagUtils.getInstance().message(bundle, userLocale, getSuffix()) != null);

        // Render the error messages appropriately
        StringBuffer results = new StringBuffer();
        boolean headerDone = false;
        String message = null;
        Iterator reports =
            (property == null) ? errorHandler.getErrors().iterator() : errorHandler.getErrorsByField(property).iterator();
            
        while (reports.hasNext()) {
            EMFAbstractError errorItem = (EMFAbstractError)reports.next();

            if (!headerDone) {
                if (headerPresent) {
                    message =
                    	MessageBundle.getMessage(getHeader(), bundle, userLocale);

                    results.append(message);
                }

                headerDone = true;
            }

            if (prefixPresent) {
                message =
                	MessageBundle.getMessage(getPrefix(), bundle, userLocale);
                results.append(message);
            }

            if ((errorItem.getDescription() != null) && (errorItem.getDescription().length() > 0)
            	&& (GenericValidator.isBlankOrNull(severity) || severity.equals(errorItem.getSeverity()))	
            	&& (GenericValidator.isBlankOrNull(category) || category.equals(errorItem.getCategory()))	
            	) {
            	// The message should be already localized
            	message = errorItem.getDescription();
            	
                if (message != null) {
                    results.append(message);
                }
            } // if ((errorItem.getDescription() != null) &&

            if (suffixPresent) {
                message =
                	MessageBundle.getMessage(getSuffix(), bundle, userLocale);
                results.append(message);
            }
        }

        if (headerDone && footerPresent) {
            message =
            	MessageBundle.getMessage(getFooter(), bundle, userLocale);
            results.append(message);
        }

        TagUtils.getInstance().write(pageContext, results.toString());

        return (EVAL_BODY_INCLUDE);
    }

    /**
     * Release any acquired resources.
     */
    public void release() {
        super.release();
        bundle = Constants.MESSAGES_KEY;
        property = null;
        header = null;
        footer = null;
        prefix = null;
        suffix = null;
    }
}
