/*
 * $Id: OptionsCollectionTag.java 376841 2006-02-10 21:01:28Z husted $
 *
 * Copyright 2002-2004 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package it.eng.spago.taglib.html;

import it.eng.spago.base.SourceBean;
import it.eng.spago.message.MessageBundleReference;
import it.eng.spago.taglib.TagUtils;

import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.TagSupport;

/**
 * Tag for creating multiple &lt;select&gt; options from a collection. The
 * collection may be part of the enclosing form, or may be independent of the
 * form. Each element (SourceBean) of the collection must expose a 'label' and
 * a 'value', the attribute names of which are configurable by attributes of
 * this tag.
 * <p>If "isLabelKey" is valorized to true the "label" is used to retrieve a
 * message from the resource bundle.</p> 
 *
 * @version $Rev: 376841 $ $Date: 2004-11-03 14:20:47 -0500 (Wed, 03 Nov 2004)
 *          $
 * @since Struts 1.1
 */
public class OptionsCollectionTag extends TagSupport {
    // ----------------------------------------------------- Instance Variables

    /**
     * The message resources for this package.
     */
    protected static MessageBundleReference messages =
        new MessageBundleReference(Constants.Package
            + ".LocalStrings");

    // ------------------------------------------------------------- Properties

    private static final long serialVersionUID = 1L;
    
    /**
     * Should the label values be filtered for HTML sensitive characters?
     */
    protected boolean filter = true;

    /**
     * The name of the bean property containing the label.
     */
    protected String label = "label";

    /**
     * @see OptionsCollectionTag#paramName
     */
    protected String listAttribute = null;
    /**
     * Say if label are to be used as keys to retrive from bundle
     */
    protected String isLabelKey = "false";
    
	/**
	 * 
	 * The attribute <code>paramName</code> is used in conjunction with
	 * the attribute <code>paramValue</code> and the attribute
	 * <code>listAttribute</code> to filter a SourceBeanAttribute.
	 * In this way is possible to navigate a SourceBean to a path that
	 * indicate a list of SourceBeanAttribute to filter; the filter is
	 * rappresented with the pair (<code>paramName</code>, <code>paramValue</code>).
	 * If the obtained attribute contain a list identified by
	 * <code>listAttribute</code>, this list is returned.
	 */
    protected String paramName = null;

    /**
     * @see OptionsCollectionTag#paramName
     */
    protected String paramValue = null;
    
    /**
     * The name of the property to use to build the values collection.
     */
    protected String property = null;
    
    /**
     * Scope used to retrive the value identified by <code>property</code>
     * attribute.
     */
    private String scope = null;

    /**
     * The style associated with this tag.
     */
    private String style = null;

    /**
     * The named style class associated with this tag.
     */
    private String styleClass = null;

    /**
     * The name of the bean property containing the value.
     */
    protected String value = "value";

    public boolean getFilter() {
        return filter;
    }

    public void setFilter(boolean filter) {
        this.filter = filter;
    }

    public String getLabel() {
        return label;
    }

    public void setLabel(String label) {
        this.label = label;
    }

    public String getProperty() {
        return property;
    }

    public void setProperty(String property) {
        this.property = property;
    }

    public String getStyle() {
        return style;
    }

    public void setStyle(String style) {
        this.style = style;
    }

    public String getStyleClass() {
        return styleClass;
    }

    public void setStyleClass(String styleClass) {
        this.styleClass = styleClass;
    }

    public String getValue() {
        return value;
    }

    public void setValue(String value) {
        this.value = value;
    }

    // --------------------------------------------------------- Public Methods

    /**
     * Process the start of this tag.
     *
     * @throws JspException if a JSP exception has occurred
     */
    public int doStartTag() throws JspException {
        // Acquire the select tag we are associated with
        SelectTag selectTag =
            (SelectTag) pageContext.getAttribute(Constants.SELECT_KEY);

        if (selectTag == null) {
            JspException e =
                new JspException(messages.getMessage(
                        "optionsCollectionTag.select"));

            TagUtils.getInstance().saveException(pageContext, e);
            throw e;
        }

        // Acquire the collection containing our options
        List collection =
            TagUtils.getInstance().lookupMultipleProperty(pageContext, property,
            	scope, paramName, paramValue, listAttribute);

        if (collection == null) {
            JspException e =
                new JspException(messages.getMessage(
                        "optionsCollectionTag.collection"));

            TagUtils.getInstance().saveException(pageContext, e);
            throw e;
        }

        // Acquire an iterator over the options collection
        Iterator iter = getIterator(collection);

        StringBuffer sb = new StringBuffer();

        // Render the options
        while (iter.hasNext()) {
            SourceBean bean = (SourceBean)iter.next();
            Object beanLabel = null;
            Object beanValue = null;
            String beanChars = bean.getCharacters();
            if (beanChars == null) { beanChars = ""; }

            // Get the label for this option
                beanLabel = (label.equals(TagUtils.CHARACTERS_KEY))?
                	bean.getCharacters() :
                	bean.getAttribute(label);

                if (beanLabel == null) {
                    beanLabel = "";
                }

            // Get the value for this option
                beanValue = (value.equals(TagUtils.CHARACTERS_KEY))?
                	bean.getCharacters() :
                	bean.getAttribute(value);

                if (beanValue == null) {
                    beanValue = "";
                }

            String stringLabel = (beanLabel instanceof SourceBean)?
            	((SourceBean)beanLabel).getCharacters() : beanLabel.toString();
            String stringValue = (beanValue instanceof SourceBean)?
                	((SourceBean)beanValue).getCharacters() : beanValue.toString();

            // Render this option
            addOption(sb, stringLabel, stringValue,
                selectTag.isMatched(stringValue));
        }

        TagUtils.getInstance().write(pageContext, sb.toString());

        return SKIP_BODY;
    }

    /**
     * Release any acquired resources.
     */
    public void release() {
        super.release();
        filter = true;
        label = "label";
        property = null;
        style = null;
        styleClass = null;
        value = "value";
    }

    // ------------------------------------------------------ Protected Methods

    /**
     * Add an option element to the specified StringBuffer based on the
     * specified parameters. <p> Note that this tag specifically does not
     * support the <code>styleId</code> tag attribute, which causes the HTML
     * <code>id</code> attribute to be emitted.  This is because the HTML
     * specification states that all "id" attributes in a document have to be
     * unique.  This tag will likely generate more than one
     * <code>option</code> element element, but it cannot use the same
     * <code>id</code> value.  It's conceivable some sort of mechanism to
     * supply an array of <code>id</code> values could be devised, but that
     * doesn't seem to be worth the trouble.
     * <p>If "isLabelKey" is valorized to true the "label" is used to retrieve a
     * message from the resource bundle.</p> 
 	 *
     * @param sb      StringBuffer accumulating our results
     * @param value   Value to be returned to the server for this option
     * @param label   Value to be shown to the user for this option
     * @param matched Should this value be marked as selected?
     * 
     */
    protected void addOption(StringBuffer sb, String label, String value,
        boolean matched) throws JspException {
        sb.append("<option value=\"");
        String resolvedLabel = label;
        
        if (Boolean.valueOf(this.isLabelKey).booleanValue()) {
        	resolvedLabel = TagUtils.getInstance().message(null, label);
        }
        
        if (filter) {
            sb.append(TagUtils.getInstance().filter(value));
        } else {
            sb.append(value);
        }

        sb.append("\"");

        if (matched) {
            sb.append(" selected=\"selected\"");
        }

        if (style != null) {
            sb.append(" style=\"");
            sb.append(style);
            sb.append("\"");
        }

        if (styleClass != null) {
            sb.append(" class=\"");
            sb.append(styleClass);
            sb.append("\"");
        }

        sb.append(">");

        if (filter) {
            sb.append(TagUtils.getInstance().filter(resolvedLabel));
        } else {
            sb.append(resolvedLabel);
        }

        sb.append("</option>\r\n");
    }

    /**
     * Return an iterator for the options collection.
     *
     * @param collection Collection to be iterated over
     * @throws JspException if an error occurs
     */
    protected Iterator getIterator(Object collection)
        throws JspException {
        if (collection.getClass().isArray()) {
            collection = Arrays.asList((Object[]) collection);
        }

        if (collection instanceof Collection) {
            return (((Collection) collection).iterator());
        } else if (collection instanceof Iterator) {
            return ((Iterator) collection);
        } else if (collection instanceof Map) {
            return (((Map) collection).entrySet().iterator());
        } else {
            throw new JspException(messages.getMessage(
                    "optionsCollectionTag.iterator", collection.toString()));
        }
    }

	public String getIsLabelKey() {
		return isLabelKey;
	}

	public void setIsLabelKey(String isLabelKeys) {
		this.isLabelKey = isLabelKeys;
	}

	public String getScope() {
		return scope;
	}

	public void setScope(String scope) {
		this.scope = scope;
	}

	public String getParamName() {
		return paramName;
	}

	public void setParamName(String paramName) {
		this.paramName = paramName;
	}

	public String getParamValue() {
		return paramValue;
	}

	public void setParamValue(String paramValue) {
		this.paramValue = paramValue;
	}

	public String getListAttribute() {
		return listAttribute;
	}

	public void setListAttribute(String listAttribute) {
		this.listAttribute = listAttribute;
	}
}
